//*****************************************************************************
//
// startup.c 
// 
// Setup the system clock and the ports.
//
//*****************************************************************************
#include <stdint.h>
#include <string.h>

#include <inc/hw_types.h>
#include <inc/hw_memmap.h>
#include <inc/hw_ssi.h>
#include <inc/hw_gpio.h>
#include <inc/hw_timer.h>
#include <inc/hw_ints.h>

#include <driverlib/gpio.h>
#include <driverlib/interrupt.h>
#include <driverlib/systick.h>
#include <driverlib/sysctl.h>
#include <driverlib/uart.h>
#include <driverlib/timer.h>

#define UART_BASE UART0_BASE
#define UART_PERIPH SYSCTL_PERIPH_UART0

#define UART_PORT_PERIPH SYSCTL_PERIPH_GPIOA
#define UART_PORT GPIO_PORTA_BASE
#define UART_BITS GPIO_PIN_0|GPIO_PIN_1

#define LED1_PERIPH SYSCTL_PERIPH_GPIOF
#define LED1_PORT GPIO_PORTF_BASE
#define LED1_BIT GPIO_PIN_0

#define CPUHZ 50000000
#define TICKHZ 1000

/*
 * Configure the processor 
 */
static void prvSetupHardware( void ) {

  	/* Setup the System Clock. 200 / 8 = 25MHz */ 
	SysCtlClockSet(SYSCTL_SYSDIV_4 |\
		SYSCTL_USE_PLL | SYSCTL_OSC_MAIN | SYSCTL_XTAL_8MHZ);

	// Enable the required GPIO Ports
	SysCtlPeripheralEnable(UART_PORT_PERIPH);  // GPIO Port for UART
	GPIOPinTypeUART(UART_PORT,UART_BITS); 

	SysCtlPeripheralEnable(LED1_PERIPH); 
	GPIOPinTypeGPIOOutput(LED1_PORT,LED1_BIT);
 	GPIOPinWrite(LED1_PORT, LED1_BIT, LED1_BIT);

	SysCtlPeripheralEnable(UART_PERIPH);  
	UARTConfigSetExpClk(UART_BASE, CPUHZ, 115200,
		(UART_CONFIG_WLEN_8 | UART_CONFIG_STOP_ONE |\
		 UART_CONFIG_PAR_NONE));

	UARTEnable(UART_BASE);

	// Turn on Clock Gating control
	SysCtlPeripheralClockGating(true);

	SysCtlPeripheralSleepEnable(LED1_PERIPH); 
	SysCtlPeripheralSleepEnable(UART_PORT_PERIPH); 
	SysCtlPeripheralSleepEnable(UART_PERIPH); 

	// Setup the Stellaris Timer to serve as 
	// an activity load measurement device.
	SysCtlPeripheralEnable(SYSCTL_PERIPH_TIMER0); 
	SysCtlPeripheralSleepDisable(SYSCTL_PERIPH_TIMER0); 
	TimerConfigure(TIMER0_BASE, TIMER_CFG_32_BIT_PER);
	TimerLoadSet(TIMER0_BASE, TIMER_A, 0xffffffff);
	TimerEnable(TIMER0_BASE, TIMER_A);

	// These two are part of core debug. 
	// NOTE!  The Cycle counter runs during sleep in 
	// early revs of the Cortex-M3.  Check the ARM Errata.
	HWREG(0xE000EDFC) = 1<<24; // Set the TRCENA Bit to turn on the DWT
	HWREG(0xE0001000) = 1; 	   // Whack CYCCNTENA 

	}

//*****************************************************************************
// Stub handler for the Serial port, so that we can use WFI
//*****************************************************************************
void UARTStubHandler() { ; }

//*****************************************************************************
// The Systick auto-rearms.  Just increment the counter.
//*****************************************************************************
unsigned int ticks_ms = 0;
unsigned int time_ms = 0;
unsigned int beforecount;
unsigned int aftercount;
unsigned int usage;

// Heres a knob for use by the delay routine.
// The delay code can can zero this out as needed.
unsigned int delayticker_ms;

void SysTickHandler() {
	ticks_ms++;
	time_ms++;
	if (time_ms == 1000) {
		time_ms = 0;
		// aftercount = HWREG(0xE0001004); // Arm Cycle Counter.
		// usage = aftercount - beforecount;

		// Load up the current value from the timer.
		aftercount = HWREG(TIMER0_BASE + TIMER_O_TAR); 
		usage = beforecount - aftercount;

		beforecount = aftercount;
		}
	delayticker_ms++;
	}

void SayHello() {
	char message[] = "Supervisor Boot!\r\n";
	char* p;

	p = message;
	while(*p != 0) { 
		UARTCharPut(UART_BASE,*p);
		p++;
		}
	}
//*****************************************************************************
// Main - Set things up, launch forth
//*****************************************************************************
// Let the C compiler know that it doesn't need to push any registers.
int main(void) __attribute__ ((naked));
int main(void) {

	// Setup the hardware.
	prvSetupHardware();

	//*****************************************************************
	// Turn on SysTick
	SysTickPeriodSet(CPUHZ/TICKHZ); 
	SysTickEnable();
	// IntPrioritySet(FAULT_SYSTICK,(1<<5));
	SysTickIntEnable();

	// Turn on all interrupts
	IntMasterEnable();

	// We can't step through a svc call.  You 
	// must set the breakpoint there instead 

	SayHello();
	// SysCallPutChar('A');

#ifdef DUALSTACK
	// So now we look up the reset vector for the new code, 
	// Initialize the Process Stack Pointer, and switch
	// to dual stack mode.
	asm(	"ldr r0,=0x1000 @ Base Address of next Image \n"
		"ldr r1, [r0]   @ This should be the stack pointer\n"
		"msr psp, r1    @ Load it into the process sp\n"
		"ldr r1,[r0, #4]@ Load up the program counter\n"
		"mov r2, 0x3	@ New value for control register\n"
		"msr CONTROL, r2@ Load it up.\n"
		"isb		@ Instruction sync barrier\n"
		"mov pc,r1	@ branch to official code\n"
		);

#else
	// Allow the client to own the whole thing.
	asm(	"ldr r0,=0x1000 @ Base Address of next Image \n"
		"ldr r1, [r0]   @ This should be the stack pointer\n"
		"mov sp, r1    @ Load it into the process sp\n"
		"ldr r1,[r0, #4]@ Load up the program counter\n"
		"mov pc,r1	@ branch to official code\n"
		);
#endif

//		"loop: b loop\n"

	// Forth owns the user mode, and the prompt.  
	// The ISRs can hijack things, but thats perfectly ok, as they will
	// save system state.

	// This code will never run.
	while(1) { ; }

	}

